"use strict";

/*---------------------------------------------------------------------------------------------
 *  Copyright (c) AlexTorresDev. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.CustomTitlebar = void 0;
const electron_1 = require("electron");
const color_1 = require("../base/common/color");
const dom_1 = require("../base/common/dom");
const platform_1 = require("../base/common/platform");
const menubar_1 = require("../menubar");
const themebar_1 = require("./themebar");
const consts_1 = require("../consts");
class CustomTitlebar extends _get__("themebar_1").ThemeBar {
  constructor(options) {
    super();
    this.isInactive = false;
    this.currentOptions = {
      backgroundColor: _get__("color_1").Color.WHITE,
      closeable: true,
      enableMnemonics: true,
      hideWhenClickingClose: true,
      iconSize: 16,
      itemBackgroundColor: undefined,
      maximizable: true,
      menuPosition: "left",
      menuTransparency: 100,
      minimizable: true,
      onlyShowMenuBar: false,
      shadow: true,
      titleHorizontalAlignment: "center"
    };
    // Temp
    this.windowIcons = `{
    "check": "<svg viewBox='0 0 11 11'><path d='M3.8,9.3c-0.1,0-0.2,0-0.3-0.1L0.2,5.8C0,5.6,0,5.4,0.2,5.2C0.4,5,0.7,5,0.9,5.2l3,3l6.3-6.3c0.2-0.2,0.5-0.2,0.7,0C11,2,11,2.3,10.8,2.5L4.2,9.1C4.1,9.2,4,9.3,3.8,9.3z'/></svg>",
    "arrow": "<svg viewBox='0 0 11 11'><path d='M3.1,10.7c-0.1,0-0.2,0-0.3-0.1c-0.2-0.2-0.2-0.5,0-0.7l4.4-4.4L2.8,1.1c-0.2-0.2-0.2-0.5,0-0.7c0.2-0.2,0.5-0.2,0.7,0l4.8,4.8c0.2,0.2,0.2,0.5,0,0.7l-4.8,4.8C3.4,10.7,3.2,10.7,3.1,10.7z'/></svg>",
    "windows": {
      "minimize": "<svg viewBox='0 0 11 11'><path d='M11,4.9v1.1H0V4.399h11z'/></svg>",
      "maximize": "<svg viewBox='0 0 11 11'><path d='M0,1.7v7.6C0,10.2,0.8,11,1.7,11h7.6c0.9,0,1.7-0.8,1.7-1.7V1.7C11,0.8,10.2,0,9.3,0H1.7C0.8,0,0,0.8,0,1.7z M8.8,9.9H2.2c-0.6,0-1.1-0.5-1.1-1.1V2.2c0-0.6,0.5-1.1,1.1-1.1h6.7c0.6,0,1.1,0.5,1.1,1.1v6.7C9.9,9.4,9.4,9.9,8.8,9.9z'/></svg>",
      "restore": "<svg viewBox='0 0 11 11'><path d='M7.9,2.2h-7C0.4,2.2,0,2.6,0,3.1v7C0,10.6,0.4,11,0.9,11h7c0.5,0,0.9-0.4,0.9-0.9v-7C8.8,2.6,8.4,2.2,7.9,2.2z M7.7,9.6 c0,0.2-0.1,0.3-0.3,0.3h-6c-0.2,0-0.3-0.1-0.3-0.3v-6c0-0.2,0.1-0.3,0.3-0.3h6c0.2,0,0.3,0.1,0.3,0.3V9.6z'/><path d='M10,0H3.5v1.1h6.1c0.2,0,0.3,0.1,0.3,0.3v6.1H11V1C11,0.4,10.6,0,10,0z'/></svg>",
      "close": "<svg viewBox='0 0 11 11'><path d='M6.279 5.5L11 10.221l-.779.779L5.5 6.279.779 11 0 10.221 4.721 5.5 0 .779.779 0 5.5 4.721 10.221 0 11 .779 6.279 5.5z'/></svg>"
    }
  }`;
    this.closeMenu = () => {
      if (this.menuBar) {
        this.menuBar.blur();
      }
    };
    this.currentOptions = {
      ...this.currentOptions,
      ...options
    };
    const jWindowsIcons = JSON.parse(this.windowIcons)[(0, _get__("platform_1").PlatformToString)(_get__("platform_1").platform).toLocaleLowerCase()];
    this.platformIcons = jWindowsIcons;
    this.titlebar = (0, _get__("dom_1").$)('.cet-titlebar');
    this.dragRegion = (0, _get__("dom_1").$)('.cet-drag-region');
    this.icon = (0, _get__("dom_1").$)('.cet-icon');
    this.menuBarContainer = (0, _get__("dom_1").$)('.cet-menubar');
    this.title = (0, _get__("dom_1").$)('.cet-title');
    this.windowControlsContainer = (0, _get__("dom_1").$)('.cet-window-controls');
    this.container = (0, _get__("dom_1").$)('.cet-container');
    this.windowControls = {
      minimize: (0, _get__("dom_1").$)('.cet-control-minimize'),
      maximize: (0, _get__("dom_1").$)('.cet-control-maximize'),
      close: (0, _get__("dom_1").$)('.cet-control-close')
    };
    this.resizer = {
      top: (0, _get__("dom_1").$)('.cet-resizer.top'),
      left: (0, _get__("dom_1").$)('.cet-resizer.left')
    };
    (0, _get__("dom_1").append)(this.titlebar, this.dragRegion);
    (0, _get__("dom_1").append)(this.titlebar, this.resizer.left);
    (0, _get__("dom_1").append)(this.titlebar, this.resizer.top);
    this.loadWindowIcons();
    this.setupBackgroundColor();
    this.createIcon();
    this.setupMenubar();
    this.setupTitle();
    this.setupWindowControls();
    this.setupContainer();
    this.setupTitleBar();
    this.loadEvents();
    //this.registerTheme(ThemeBar.win)
  }

  loadWindowIcons() {
    const windowIcons = this.currentOptions.icons;
    if (!windowIcons) return;
    const jWindowsIcons = require(windowIcons);
    this.platformIcons = jWindowsIcons[(0, _get__("platform_1").PlatformToString)(_get__("platform_1").platform).toLocaleLowerCase()];
  }
  setupBackgroundColor() {
    let color = this.currentOptions.backgroundColor;
    if (!color) {
      const metaColor = document.querySelectorAll('meta[name="theme-color"]') || document.querySelectorAll('meta[name="msapplication-TileColor"]');
      metaColor.forEach(meta => {
        color = _get__("color_1").Color.fromHex(meta.getAttribute('content'));
      });
      this.currentOptions.backgroundColor = color;
    }
    this.titlebar.style.backgroundColor = color.toString();
  }
  createIcon() {
    const onlyRendererMenuBar = this.currentOptions.onlyShowMenuBar;
    if (_get__("platform_1").isMacintosh || onlyRendererMenuBar) return;
    let icon = this.currentOptions.icon;
    if (!icon) {
      const tagLink = document.querySelectorAll('link');
      tagLink.forEach(link => {
        if (link.getAttribute('rel') === 'icon' || link.getAttribute('rel') === 'shortcut icon') {
          icon = link.getAttribute('href');
        }
        this.currentOptions.icon = icon;
      });
    }
    if (icon) {
      const windowIcon = (0, _get__("dom_1").append)(this.icon, (0, _get__("dom_1").$)('img'));
      if (typeof icon === 'string') {
        windowIcon.setAttribute('src', icon);
      } else {
        windowIcon.setAttribute('src', icon.toDataURL());
      }
      this.setIconSize(this.currentOptions.iconSize);
      (0, _get__("dom_1").append)(this.titlebar, this.icon);
    }
  }
  setIconSize(size) {
    if (size < 16) size = 16;
    if (size > 24) size = 24;
    this.icon.firstElementChild.setAttribute('style', `height: ${size}px`);
  }
  setupMenubar() {
    _get__("electron_1").ipcRenderer.invoke('request-application-menu').then(menu => this.updateMenu(menu));
    const menuPosition = this.currentOptions.menuPosition;
    if (menuPosition) {
      this.updateMenuPosition(menuPosition);
    }
    (0, _get__("dom_1").append)(this.titlebar, this.menuBarContainer);
  }
  setupTitle() {
    const onlyRendererMenuBar = this.currentOptions.onlyShowMenuBar;
    if (onlyRendererMenuBar) return;
    this.updateTitle(document.title);
    this.updateTitleAlignment(this.currentOptions.titleHorizontalAlignment);
    (0, _get__("dom_1").append)(this.titlebar, this.title);
  }
  createControlButton(element, icon, active = true) {
    (0, _get__("dom_1").addClass)(element, 'cet-control-icon');
    element.innerHTML = icon;
    if (!active) {
      (0, _get__("dom_1").addClass)(element, 'inactive');
    }
    (0, _get__("dom_1").append)(this.windowControlsContainer, element);
  }
  setupWindowControls() {
    const onlyRendererMenuBar = this.currentOptions.onlyShowMenuBar;
    if (_get__("platform_1").isMacintosh || onlyRendererMenuBar) return;
    const order = this.currentOptions.order;
    if (order === 'inverted') {
      this.windowControlsContainer.style.flexDirection = 'row-reverse';
    }
    this.createControlButton(this.windowControls.minimize, this.platformIcons.minimize, this.currentOptions.minimizable);
    this.createControlButton(this.windowControls.maximize, this.platformIcons.maximize, this.currentOptions.maximizable);
    this.createControlButton(this.windowControls.close, this.platformIcons.close, this.currentOptions.closeable);
    (0, _get__("dom_1").append)(this.titlebar, this.windowControlsContainer);
  }
  setupContainer() {
    while (document.body.firstChild) {
      (0, _get__("dom_1").append)(this.container, document.body.firstChild);
    }
    this.container.style.top = _get__("consts_1").TOP_TITLEBAR_HEIGHT_WIN;
    (0, _get__("dom_1").append)(document.body, this.container);
  }
  setupTitleBar() {
    const order = this.currentOptions.order;
    const hasShadow = this.currentOptions.shadow;
    (0, _get__("dom_1").addClass)(this.titlebar, `cet-${(0, _get__("platform_1").PlatformToString)(_get__("platform_1").platform).toLocaleLowerCase()}`);
    if (order) {
      (0, _get__("dom_1").addClass)(this.titlebar, `cet-${order}`);
    }
    if (hasShadow) {
      (0, _get__("dom_1").addClass)(this.titlebar, 'cet-shadow');
    }
    if (!_get__("platform_1").isMacintosh) {
      this.title.style.cursor = 'default';
    }
    (0, _get__("dom_1").prepend)(document.body, this.titlebar);
  }
  loadEvents() {
    const onlyRendererMenuBar = this.currentOptions.onlyShowMenuBar;
    if (onlyRendererMenuBar) return;
    const minimizable = this.currentOptions.minimizable;
    const maximizable = this.currentOptions.maximizable;
    const closeable = this.currentOptions.closeable;
    _get__("electron_1").ipcRenderer.on('window-maximize', (_, isMaximized) => this.onDidChangeMaximized(isMaximized));
    _get__("electron_1").ipcRenderer.on('window-minimize', (_, isFullScreen) => this.onWindowFullScreen(isFullScreen));
    _get__("electron_1").ipcRenderer.on('window-focus', (_, isFocused) => this.onWindowFocus(isFocused));
    if (minimizable) {
      (0, _get__("dom_1").addDisposableListener)(this.windowControls.minimize, _get__("dom_1").EventType.CLICK, () => {
        _get__("electron_1").ipcRenderer.send('window-event', 'window-minimize');
      });
    }
    if (_get__("platform_1").isMacintosh) {
      (0, _get__("dom_1").addDisposableListener)(this.titlebar, _get__("dom_1").EventType.DBLCLICK, () => {
        _get__("electron_1").ipcRenderer.send('window-event', 'window-maximize');
      });
    }
    if (maximizable) {
      (0, _get__("dom_1").addDisposableListener)(this.windowControls.maximize, _get__("dom_1").EventType.CLICK, () => {
        _get__("electron_1").ipcRenderer.send('window-event', 'window-maximize');
      });
    }
    if (closeable) {
      (0, _get__("dom_1").addDisposableListener)(this.windowControls.close, _get__("dom_1").EventType.CLICK, () => {
        _get__("electron_1").ipcRenderer.send('window-event', 'window-close');
      });
    }
  }
  /** X */
  onBlur() {
    this.isInactive = true;
    // this.updateStyles()
  }

  onFocus() {
    this.isInactive = false;
    // this.updateStyles()
  }

  onMenubarVisibilityChanged(visible) {
    if (_get__("platform_1").isWindows || _get__("platform_1").isLinux) {
      if (visible) {
        // Hack to fix issue #52522 with layered webkit-app-region elements appearing under cursor
        (0, _get__("dom_1").hide)(this.dragRegion);
        setTimeout(() => (0, _get__("dom_1").show)(this.dragRegion), 50);
      }
    }
  }
  onMenubarFocusChanged(focused) {
    if (_get__("platform_1").isWindows || _get__("platform_1").isLinux) {
      if (focused) (0, _get__("dom_1").hide)(this.dragRegion);else (0, _get__("dom_1").show)(this.dragRegion);
    }
  }
  onDidChangeMaximized(isMaximized) {
    const maximize = this.windowControls.maximize;
    if (maximize) {
      maximize.title = isMaximized ? "Restore Down" : "Maximize";
      maximize.innerHTML = isMaximized ? this.platformIcons['restore'] : this.platformIcons['maximize'];
    }
    if (this.resizer) {
      if (isMaximized) (0, _get__("dom_1").hide)(this.resizer.top, this.resizer.left);else (0, _get__("dom_1").show)(this.resizer.top, this.resizer.left);
    }
  }
  updateMenu(menu) {
    if (!_get__("platform_1").isMacintosh) {
      //if (this.menuBar) this.menuBar.dispose()
      if (!menu) return;
      this.menuBar = new (_get__("menubar_1").MenuBar)(this.menuBarContainer, this.currentOptions, menu, this.closeMenu);
      // this.menuBar.setupMenubar()
      this.menuBar.onVisibilityChange(e => this.onMenubarVisibilityChanged(e));
      this.menuBar.onFocusStateChange(e => this.onMenubarFocusChanged(e));
      //this._updateStyles()
    }
  }
  /// Public methods
  /**
   * Update title bar styles based on focus state.
   * @param hasFocus focus state of the window
   */
  onWindowFocus(focus) {
    if (this.titlebar) {
      if (focus) {
        (0, _get__("dom_1").removeClass)(this.titlebar, 'inactive');
        this.onFocus();
      } else {
        (0, _get__("dom_1").addClass)(this.titlebar, 'inactive');
        // this.closeMenu()
        this.onBlur();
      }
    }
  }
  /**
   * Update the full screen state and hide or show the title bar.
   * @param fullscreen Fullscreen state of the window
   */
  onWindowFullScreen(fullscreen) {
    if (!_get__("platform_1").isMacintosh) {
      if (fullscreen) {
        (0, _get__("dom_1").hide)(this.titlebar);
        this.container.style.top = '0px';
      } else {
        (0, _get__("dom_1").show)(this.titlebar);
        if (this.currentOptions.menuPosition === 'bottom') this.container.style.top = _get__("consts_1").BOTTOM_TITLEBAR_HEIGHT;else this.container.style.top = _get__("platform_1").isMacintosh ? _get__("consts_1").TOP_TITLEBAR_HEIGHT_MAC : _get__("consts_1").TOP_TITLEBAR_HEIGHT_WIN;
      }
    }
  }
  /**
   * Update the title of the title bar.
   * You can use this method if change the content of `<title>` tag on your html.
   * @param title The title of the title bar and document.
   */
  updateTitle(title) {
    this.title.innerText = title;
    document.title = title;
    return this;
  }
  /**
   * It method set new icon to title-bar-icon of title-bar.
   * @param path path to icon
   */
  updateIcon(path) {
    if (this.icon) {
      this.icon.firstElementChild.setAttribute('src', path);
    }
    return this;
  }
  /**
   * Horizontal alignment of the title.
   * @param side `left`, `center` or `right`.
   */
  updateTitleAlignment(side) {
    const order = this.currentOptions.order;
    const menuPosition = this.currentOptions.menuPosition;
    (0, _get__("dom_1").removeClass)(this.title, 'cet-title-left');
    (0, _get__("dom_1").removeClass)(this.title, 'cet-title-right');
    (0, _get__("dom_1").removeClass)(this.title, 'cet-title-center');
    if (side === 'left' || side === 'right' && order === 'inverted') {
      (0, _get__("dom_1").addClass)(this.title, 'cet-title-left');
    }
    if (side === 'right' || side === 'left' && order === 'inverted') {
      (0, _get__("dom_1").addClass)(this.title, 'cet-title-right');
    }
    if (!side || side === 'center') {
      if (menuPosition !== 'bottom') {
        /* addDisposableListener(window, 'resize', () => {
          if (window.innerWidth >= 1188) {
            addClass(this.title, 'cet-title-center')
          } else {
            removeClass(this.title, 'cet-title-center')
          }
        }) */
        (0, _get__("dom_1").addClass)(this.title, 'cet-title-center');
      }
      if (!_get__("platform_1").isMacintosh && order === 'first-buttons') {
        this.windowControlsContainer.style.marginLeft = 'auto';
      }
      this.title.style.maxWidth = 'calc(100% - 296px)';
    }
    return this;
  }
  /**
   * Update the background color of the title bar
   * @param backgroundColor The color for the background
   */
  updateBackground(backgroundColor) {
    this.currentOptions.backgroundColor = backgroundColor;
    // this.updateStyles()
    return this;
  }
  /**
   * Update the item background color of the menubar
   * @param itemBGColor The color for the item background
   */
  updateItemBGColor(itemBGColor) {
    this.currentOptions.itemBackgroundColor = itemBGColor;
    // this._updateStyles()
    return this;
  }
  /**
   * Update the menu from Menu.getApplicationMenu()
   */
  async refreshMenu() {
    if (!_get__("platform_1").isMacintosh) {
      _get__("electron_1").ipcRenderer.invoke('request-application-menu').then(menu => this.updateMenu(menu));
    }
    return this;
  }
  /**
   * Update the position of menubar.
   * @param menuPosition The position of the menu `left` or `bottom`.
   */
  updateMenuPosition(menuPosition) {
    const height = _get__("platform_1").isMacintosh ? _get__("consts_1").TOP_TITLEBAR_HEIGHT_MAC : _get__("consts_1").TOP_TITLEBAR_HEIGHT_WIN;
    this.currentOptions.menuPosition = menuPosition;
    this.titlebar.style.height = menuPosition === 'bottom' ? _get__("consts_1").BOTTOM_TITLEBAR_HEIGHT : height;
    this.container.style.top = menuPosition === 'bottom' ? _get__("consts_1").BOTTOM_TITLEBAR_HEIGHT : height;
    if (menuPosition === 'bottom') {
      (0, _get__("dom_1").addClass)(this.menuBarContainer, 'bottom');
    } else {
      (0, _get__("dom_1").removeClass)(this.menuBarContainer, 'bottom');
    }
    return this;
  }
  /**
   * Remove the titlebar, menubar and all methods.
   */
  dispose() {
    // if (this.menuBar) this.menuBar.dispose()
    this.titlebar.remove();
    while (this.container.firstChild) (0, _get__("dom_1").append)(document.body, this.container.firstChild);
    this.container.remove();
  }
}
exports.CustomTitlebar = _get__("CustomTitlebar");
function _getGlobalObject() {
  try {
    if (!!global) {
      return global;
    }
  } catch (e) {
    try {
      if (!!window) {
        return window;
      }
    } catch (e) {
      return this;
    }
  }
}
;
var _RewireModuleId__ = null;
function _getRewireModuleId__() {
  if (_RewireModuleId__ === null) {
    let globalVariable = _getGlobalObject();
    if (!globalVariable.__$$GLOBAL_REWIRE_NEXT_MODULE_ID__) {
      globalVariable.__$$GLOBAL_REWIRE_NEXT_MODULE_ID__ = 0;
    }
    _RewireModuleId__ = __$$GLOBAL_REWIRE_NEXT_MODULE_ID__++;
  }
  return _RewireModuleId__;
}
function _getRewireRegistry__() {
  let theGlobalVariable = _getGlobalObject();
  if (!theGlobalVariable.__$$GLOBAL_REWIRE_REGISTRY__) {
    theGlobalVariable.__$$GLOBAL_REWIRE_REGISTRY__ = Object.create(null);
  }
  return theGlobalVariable.__$$GLOBAL_REWIRE_REGISTRY__;
}
function _getRewiredData__() {
  let moduleId = _getRewireModuleId__();
  let registry = _getRewireRegistry__();
  let rewireData = registry[moduleId];
  if (!rewireData) {
    registry[moduleId] = Object.create(null);
    rewireData = registry[moduleId];
  }
  return rewireData;
}
(function registerResetAll() {
  let theGlobalVariable = _getGlobalObject();
  if (!theGlobalVariable['__rewire_reset_all__']) {
    theGlobalVariable['__rewire_reset_all__'] = function () {
      theGlobalVariable.__$$GLOBAL_REWIRE_REGISTRY__ = Object.create(null);
    };
  }
})();
var INTENTIONAL_UNDEFINED = '__INTENTIONAL_UNDEFINED__';
let _RewireAPI__ = {};
(function () {
  function addPropertyToAPIObject(name, value) {
    Object.defineProperty(_RewireAPI__, name, {
      value: value,
      enumerable: false,
      configurable: true
    });
  }
  addPropertyToAPIObject('__get__', _get__);
  addPropertyToAPIObject('__GetDependency__', _get__);
  addPropertyToAPIObject('__Rewire__', _set__);
  addPropertyToAPIObject('__set__', _set__);
  addPropertyToAPIObject('__reset__', _reset__);
  addPropertyToAPIObject('__ResetDependency__', _reset__);
  addPropertyToAPIObject('__with__', _with__);
})();
function _get__(variableName) {
  let rewireData = _getRewiredData__();
  if (rewireData[variableName] === undefined) {
    return _get_original__(variableName);
  } else {
    var value = rewireData[variableName];
    if (value === INTENTIONAL_UNDEFINED) {
      return undefined;
    } else {
      return value;
    }
  }
}
function _get_original__(variableName) {
  switch (variableName) {
    case "color_1":
      return color_1;
    case "platform_1":
      return platform_1;
    case "dom_1":
      return dom_1;
    case "electron_1":
      return electron_1;
    case "consts_1":
      return consts_1;
    case "menubar_1":
      return menubar_1;
    case "themebar_1":
      return themebar_1;
    case "CustomTitlebar":
      return CustomTitlebar;
  }
  return undefined;
}
function _assign__(variableName, value) {
  let rewireData = _getRewiredData__();
  if (rewireData[variableName] === undefined) {
    return _set_original__(variableName, value);
  } else {
    return rewireData[variableName] = value;
  }
}
function _set_original__(variableName, _value) {
  switch (variableName) {}
  return undefined;
}
function _update_operation__(operation, variableName, prefix) {
  var oldValue = _get__(variableName);
  var newValue = operation === '++' ? oldValue + 1 : oldValue - 1;
  _assign__(variableName, newValue);
  return prefix ? newValue : oldValue;
}
function _set__(variableName, value) {
  let rewireData = _getRewiredData__();
  if (typeof variableName === 'object') {
    Object.keys(variableName).forEach(function (name) {
      rewireData[name] = variableName[name];
    });
    return function () {
      Object.keys(variableName).forEach(function (name) {
        _reset__(variableName);
      });
    };
  } else {
    if (value === undefined) {
      rewireData[variableName] = INTENTIONAL_UNDEFINED;
    } else {
      rewireData[variableName] = value;
    }
    return function () {
      _reset__(variableName);
    };
  }
}
function _reset__(variableName) {
  let rewireData = _getRewiredData__();
  delete rewireData[variableName];
  if (Object.keys(rewireData).length == 0) {
    delete _getRewireRegistry__()[_getRewireModuleId__];
  }
  ;
}
function _with__(object) {
  let rewireData = _getRewiredData__();
  var rewiredVariableNames = Object.keys(object);
  var previousValues = {};
  function reset() {
    rewiredVariableNames.forEach(function (variableName) {
      rewireData[variableName] = previousValues[variableName];
    });
  }
  return function (callback) {
    rewiredVariableNames.forEach(function (variableName) {
      previousValues[variableName] = rewireData[variableName];
      rewireData[variableName] = object[variableName];
    });
    let result = callback();
    if (!!result && typeof result.then == 'function') {
      result.then(reset).catch(reset);
    } else {
      reset();
    }
    return result;
  };
}
let _typeOfOriginalExport = typeof module.exports;
function addNonEnumerableProperty(name, value) {
  Object.defineProperty(module.exports, name, {
    value: value,
    enumerable: false,
    configurable: true
  });
}
if ((_typeOfOriginalExport === 'object' || _typeOfOriginalExport === 'function') && Object.isExtensible(module.exports)) {
  addNonEnumerableProperty('__get__', _get__);
  addNonEnumerableProperty('__GetDependency__', _get__);
  addNonEnumerableProperty('__Rewire__', _set__);
  addNonEnumerableProperty('__set__', _set__);
  addNonEnumerableProperty('__reset__', _reset__);
  addNonEnumerableProperty('__ResetDependency__', _reset__);
  addNonEnumerableProperty('__with__', _with__);
  addNonEnumerableProperty('__RewireAPI__', _RewireAPI__);
}